
# q-expansion of newform 7350.2.a.bi, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7350
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [1], [0], [0], [2], [-1], [3], [0], [1], [-5], [-7], [2], [-7], [11], [8], [1], [5], [3], [12], [12], [-6], [10], [-11], [10], [-2], [-12], [-11], [2], [0], [6], [22], [8], [12], [-20], [15], [-18], [-2], [1], [18], [24], [-10], [18], [-3], [-14], [17], [0], [-13], [9], [-17], [10], [-24], [0], [-22], [-7], [23], [21], [10], [28], [2], [32], [-16], [24], [-2], [18], [-6], [-3], [17], [27], [2], [-15], [-6], [25], [23], [-24], [-25], [4], [10], [13], [-8], [10], [-25], [22], [-3], [14], [-35], [16], [20], [-33], [-12], [-4], [33], [20], [2], [2], [25], [-6], [-20], [3], [4], [32], [37], [-18], [-31], [30], [7], [38], [-27], [14], [5], [8], [28], [-14], [-38], [10], [-38], [42], [34], [-22], [-14], [-30], [-42], [-29], [-32], [46], [-42], [27], [20], [10], [13], [-11], [-25], [-29], [32], [2], [18], [20], [24], [-22], [-42], [0], [18], [-18], [-54], [2], [5], [40], [29], [-22], [0], [16], [32], [-7], [-39], [58], [17], [-53], [0], [45], [-42], [48], [52], [16], [-8], [-12], [-18], [-16], [-8], [58], [-25], [24], [-30], [53], [-43], [31], [-20], [-35], [-28], [-23], [-11], [35], [42], [-47], [6], [-47], [-24], [30], [-8], [-6], [25], [48], [-36], [-24], [32], [38], [32], [49], [-43], [46], [-62], [-16], [50], [-17], [-47], [-70], [65], [-12], [5], [-34], [30], [-62], [22], [-22], [-34], [-27], [-50], [8], [-8], [-42], [-33], [1], [32], [-35], [10], [-4], [-12], [10], [-26], [-40], [28], [62], [-4], [20], [-68], [22], [-56], [8], [10], [-54], [-10], [-62], [-24], [-62], [-1], [-10], [39], [0], [53], [28], [60], [-24], [-68], [-27], [-33], [-10], [24], [-10], [-58], [38], [-62], [-2], [-24], [-48], [-35], [-66], [23], [-10], [75], [-7], [-49], [-59], [-7], [-3], [-56], [0], [-12], [-36], [-48], [50], [-73], [33], [44], [-28], [-12], [-37], [-32], [-3], [11], [-78], [-25], [50], [-38], [-12], [36], [-32], [46], [0], [23], [-66], [-75], [-42], [44], [57], [-10], [-24], [52], [-43], [22], [25], [-45], [26], [-6], [-25], [42], [16], [-3], [-50], [20], [-63], [44], [80], [78], [62], [78], [16], [66], [28], [-45], [74], [-48], [21], [-78], [37], [15], [-21], [-33], [28], [-20], [-21], [38], [-7], [-34], [42], [60], [-8], [-49], [0], [-22], [17], [-12], [-2], [-52], [-13], [-63], [79], [-30], [-51], [-30], [3], [-78], [-19], [-28], [-2], [-48], [-40], [-42], [1], [-22], [-24], [57], [72], [60], [81], [-15], [-47], [-28], [-15], [-8], [84], [-85], [-32], [-42], [96], [12], [-48], [-40], [24], [-58], [-42], [-4], [-22], [-65], [84], [-90], [38], [-43], [-14], [35], [-45], [12], [102], [-85], [-54], [-48], [33], [60], [-57], [22], [42], [54], [0], [84], [-58], [-24], [-13], [27], [8], [85], [3], [23], [-106], [85], [43], [27], [-105], [44], [-52], [86], [90], [68], [-90], [98], [-18], [-95], [-66], [23], [-102], [-45], [52], [92], [40], [-51], [-60], [-60], [25], [68], [32], [-86], [-42], [20], [-27], [-83], [-92], [6], [-5], [-63], [2], [30], [-77], [29], [-38], [60], [-38], [20], [102], [-82], [6], [-90], [-41], [-10], [68], [66], [82], [60], [97]]
